@@ -1,7 +1,7 @@
 /* GStreamer
  * Copyright (C) 2012 Olivier Crete <olivier.crete@collabora.com>
  *
- * gstv4l2deviceprovider.c: V4l2 device probing and monitoring
+ * pulsedeviceprovider.c: pulseaudio device probing and monitoring
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -40,7 +40,7 @@ GST_DEBUG_CATEGORY_EXTERN (pulse_debug);
 
 static GstDevice *gst_pulse_device_new (guint id,
     const gchar * device_name, GstCaps * caps, const gchar * internal_name,
-    GstPulseDeviceType type);
+    GstPulseDeviceType type, GstStructure * properties, gboolean is_default);
 
 G_DEFINE_TYPE (GstPulseDeviceProvider, gst_pulse_device_provider,
     GST_TYPE_DEVICE_PROVIDER);
@@ -65,6 +65,12 @@ enum
 };
 
 
+typedef struct
+{
+  GList *devices;
+  GstPulseDeviceProvider *self;
+} ListDevicesData;
+
 static void
 gst_pulse_device_provider_class_init (GstPulseDeviceProviderClass * klass)
 {
@@ -114,6 +120,8 @@ gst_pulse_device_provider_finalize (GObject * object)
 
   g_free (self->client_name);
   g_free (self->server);
+  g_free (self->default_sink_name);
+  g_free (self->default_source_name);
 
   G_OBJECT_CLASS (gst_pulse_device_provider_parent_class)->finalize (object);
 }
@@ -186,9 +194,10 @@ context_state_cb (pa_context * c, void *userdata)
 }
 
 static GstDevice *
-new_source (const pa_source_info * info)
+new_source (GstPulseDeviceProvider * self, const pa_source_info * info)
 {
   GstCaps *caps;
+  GstStructure *props;
   guint i;
 
   caps = gst_caps_new_empty ();
@@ -196,14 +205,18 @@ new_source (const pa_source_info * info)
   for (i = 0; i < info->n_formats; i++)
     gst_caps_append (caps, gst_pulse_format_info_to_caps (info->formats[i]));
 
+  props = gst_pulse_make_structure (info->proplist);
+
   return gst_pulse_device_new (info->index, info->description,
-      caps, info->name, GST_PULSE_DEVICE_TYPE_SOURCE);
+      caps, info->name, GST_PULSE_DEVICE_TYPE_SOURCE, props,
+      !g_strcmp0 (info->name, self->default_source_name));
 }
 
 static GstDevice *
-new_sink (const pa_sink_info * info)
+new_sink (GstPulseDeviceProvider * self, const pa_sink_info * info)
 {
   GstCaps *caps;
+  GstStructure *props;
   guint i;
 
   caps = gst_caps_new_empty ();
@@ -211,8 +224,11 @@ new_sink (const pa_sink_info * info)
   for (i = 0; i < info->n_formats; i++)
     gst_caps_append (caps, gst_pulse_format_info_to_caps (info->formats[i]));
 
+  props = gst_pulse_make_structure (info->proplist);
+
   return gst_pulse_device_new (info->index, info->description,
-      caps, info->name, GST_PULSE_DEVICE_TYPE_SINK);
+      caps, info->name, GST_PULSE_DEVICE_TYPE_SINK, props,
+      !g_strcmp0 (info->name, self->default_sink_name));
 }
 
 static void
@@ -227,12 +243,68 @@ get_source_info_cb (pa_context * context,
     return;
   }
 
-  dev = new_source (info);
+  dev = new_source (self, info);
 
   if (dev)
     gst_device_provider_device_add (GST_DEVICE_PROVIDER (self), dev);
 }
 
+static void
+get_server_info_cb (pa_context * context, const pa_server_info * info,
+    void *userdata)
+{
+  GList *tmp, *devices = NULL;
+  GstPulseDeviceProvider *self = userdata;
+
+  GST_OBJECT_LOCK (self);
+  g_free (self->default_sink_name);
+  g_free (self->default_source_name);
+  self->default_sink_name = g_strdup (info->default_sink_name);
+  self->default_source_name = g_strdup (info->default_source_name);
+  GST_DEBUG_OBJECT (self, "Default sink name: %s", self->default_sink_name);
+
+  for (tmp = GST_DEVICE_PROVIDER (self)->devices; tmp; tmp = tmp->next)
+    devices = g_list_prepend (devices, gst_object_ref (tmp->data));
+  GST_OBJECT_UNLOCK (self);
+
+  for (tmp = devices; tmp; tmp = tmp->next) {
+    GstPulseDevice *dev = tmp->data;
+    GstStructure *props = gst_device_get_properties (GST_DEVICE (dev));
+    gboolean was_default = FALSE, is_default = FALSE;
+
+    g_assert (props);
+    gst_structure_get_boolean (props, "is-default", &was_default);
+    switch (dev->type) {
+      case GST_PULSE_DEVICE_TYPE_SINK:
+        is_default = !g_strcmp0 (dev->internal_name, self->default_sink_name);
+        break;
+      case GST_PULSE_DEVICE_TYPE_SOURCE:
+        is_default = !g_strcmp0 (dev->internal_name, self->default_source_name);
+        break;
+    }
+
+    if (was_default != is_default) {
+      GstDevice *updated_device;
+      gchar *name = gst_device_get_display_name (GST_DEVICE (dev));
+
+      gst_structure_set (props, "is-default", G_TYPE_BOOLEAN, is_default, NULL);
+      updated_device = gst_pulse_device_new (dev->device_index,
+          name, gst_device_get_caps (GST_DEVICE (dev)), dev->internal_name,
+          dev->type, props, is_default);
+
+      gst_device_provider_device_changed (GST_DEVICE_PROVIDER (self),
+          updated_device, GST_DEVICE (dev));
+
+      g_free (name);
+    } else {
+      gst_structure_free (props);
+    }
+  }
+  g_list_free_full (devices, gst_object_unref);
+
+  pa_threaded_mainloop_signal (self->mainloop, 0);
+}
+
 static void
 get_sink_info_cb (pa_context * context,
     const pa_sink_info * info, int eol, void *userdata)
@@ -245,7 +317,7 @@ get_sink_info_cb (pa_context * context,
     return;
   }
 
-  dev = new_sink (info);
+  dev = new_sink (self, info);
 
   if (dev)
     gst_device_provider_device_add (GST_DEVICE_PROVIDER (self), dev);
@@ -262,6 +334,13 @@ context_subscribe_cb (pa_context * context, pa_subscription_event_type_t type,
   pa_subscription_event_type_t event_type =
       type & PA_SUBSCRIPTION_EVENT_TYPE_MASK;
 
+  if (facility == PA_SUBSCRIPTION_EVENT_SERVER ||
+      facility != PA_SUBSCRIPTION_EVENT_CHANGE) {
+    pa_context_get_server_info (self->context, get_server_info_cb, self);
+
+    return;
+  }
+
   if (facility != PA_SUBSCRIPTION_EVENT_SOURCE &&
       facility != PA_SUBSCRIPTION_EVENT_SINK)
     return;
@@ -306,34 +385,38 @@ static void
 get_source_info_list_cb (pa_context * context, const pa_source_info * info,
     int eol, void *userdata)
 {
-  GList **devices = userdata;
+  ListDevicesData *data = userdata;
 
   if (eol)
     return;
 
-  *devices = g_list_prepend (*devices, gst_object_ref_sink (new_source (info)));
+  data->devices =
+      g_list_prepend (data->devices,
+      gst_object_ref_sink (new_source (data->self, info)));
 }
 
 static void
 get_sink_info_list_cb (pa_context * context, const pa_sink_info * info,
     int eol, void *userdata)
 {
-  GList **devices = userdata;
+  ListDevicesData *data = userdata;
 
   if (eol)
     return;
 
-  *devices = g_list_prepend (*devices, gst_object_ref_sink (new_sink (info)));
+  data->devices =
+      g_list_prepend (data->devices, gst_object_ref_sink (new_sink (data->self,
+              info)));
 }
 
 static GList *
 gst_pulse_device_provider_probe (GstDeviceProvider * provider)
 {
   GstPulseDeviceProvider *self = GST_PULSE_DEVICE_PROVIDER (provider);
-  GList *devices = NULL;
   pa_mainloop *m = NULL;
   pa_context *c = NULL;
   pa_operation *o;
+  ListDevicesData data = { NULL, self };
 
   if (!(m = pa_mainloop_new ()))
     return NULL;
@@ -370,7 +453,7 @@ gst_pulse_device_provider_probe (GstDeviceProvider * provider)
   }
   GST_DEBUG_OBJECT (self, "connected");
 
-  o = pa_context_get_sink_info_list (c, get_sink_info_list_cb, &devices);
+  o = pa_context_get_sink_info_list (c, get_sink_info_list_cb, &data);
   while (pa_operation_get_state (o) == PA_OPERATION_RUNNING &&
       pa_operation_get_state (o) == PA_OPERATION_RUNNING) {
     if (pa_mainloop_iterate (m, TRUE, NULL) < 0)
@@ -378,7 +461,7 @@ gst_pulse_device_provider_probe (GstDeviceProvider * provider)
   }
   pa_operation_unref (o);
 
-  o = pa_context_get_source_info_list (c, get_source_info_list_cb, &devices);
+  o = pa_context_get_source_info_list (c, get_source_info_list_cb, &data);
   while (pa_operation_get_state (o) == PA_OPERATION_RUNNING &&
       pa_operation_get_state (o) == PA_OPERATION_RUNNING) {
     if (pa_mainloop_iterate (m, TRUE, NULL) < 0)
@@ -389,18 +472,38 @@ gst_pulse_device_provider_probe (GstDeviceProvider * provider)
   pa_context_disconnect (c);
   pa_mainloop_free (m);
 
-  return devices;
+  return data.devices;
 
 failed:
 
   return NULL;
 }
 
+static gboolean
+run_pulse_operation (GstPulseDeviceProvider * self, pa_operation * operation)
+{
+  if (!operation)
+    return FALSE;
+
+  while (pa_operation_get_state (operation) == PA_OPERATION_RUNNING) {
+    if (!PA_CONTEXT_IS_GOOD (pa_context_get_state ((self->context)))) {
+      pa_operation_cancel (operation);
+      pa_operation_unref (operation);
+      return FALSE;
+    }
+
+    pa_threaded_mainloop_wait (self->mainloop);
+  }
+
+  pa_operation_unref (operation);
+
+  return TRUE;
+}
+
 static gboolean
 gst_pulse_device_provider_start (GstDeviceProvider * provider)
 {
   GstPulseDeviceProvider *self = GST_PULSE_DEVICE_PROVIDER (provider);
-  pa_operation *initial_operation;
 
   if (!(self->mainloop = pa_threaded_mainloop_new ())) {
     GST_ERROR_OBJECT (self, "Could not create pulseaudio mainloop");
@@ -454,29 +557,21 @@ gst_pulse_device_provider_start (GstDeviceProvider * provider)
   GST_DEBUG_OBJECT (self, "connected");
 
   pa_context_subscribe (self->context,
-      PA_SUBSCRIPTION_MASK_SOURCE | PA_SUBSCRIPTION_MASK_SINK, NULL, NULL);
+      PA_SUBSCRIPTION_MASK_SOURCE | PA_SUBSCRIPTION_MASK_SINK |
+      PA_SUBSCRIPTION_EVENT_SERVER | PA_SUBSCRIPTION_EVENT_CHANGE, NULL, NULL);
 
-  initial_operation = pa_context_get_source_info_list (self->context,
-      get_source_info_cb, self);
-  while (pa_operation_get_state (initial_operation) == PA_OPERATION_RUNNING) {
-    if (!PA_CONTEXT_IS_GOOD (pa_context_get_state ((self->context))))
-      goto cancel_and_fail;
-
-    pa_threaded_mainloop_wait (self->mainloop);
-  }
-  pa_operation_unref (initial_operation);
-
-  initial_operation = pa_context_get_sink_info_list (self->context,
-      get_sink_info_cb, self);
-  if (!initial_operation)
+  if (!run_pulse_operation (self, pa_context_get_server_info (self->context,
+              get_server_info_cb, self)))
     goto unlock_and_fail;
-  while (pa_operation_get_state (initial_operation) == PA_OPERATION_RUNNING) {
-    if (!PA_CONTEXT_IS_GOOD (pa_context_get_state ((self->context))))
-      goto cancel_and_fail;
 
-    pa_threaded_mainloop_wait (self->mainloop);
-  }
-  pa_operation_unref (initial_operation);
+  if (!run_pulse_operation (self,
+          pa_context_get_source_info_list (self->context, get_source_info_cb,
+              self)))
+    goto unlock_and_fail;
+
+  if (!run_pulse_operation (self, pa_context_get_sink_info_list (self->context,
+              get_sink_info_cb, self)))
+    goto unlock_and_fail;
 
   pa_threaded_mainloop_unlock (self->mainloop);
 
@@ -489,11 +584,6 @@ unlock_and_fail:
 
 mainloop_failed:
   return FALSE;
-
-cancel_and_fail:
-  pa_operation_cancel (initial_operation);
-  pa_operation_unref (initial_operation);
-  goto unlock_and_fail;
 }
 
 static void
@@ -601,9 +691,11 @@ gst_pulse_device_reconfigure_element (GstDevice * device, GstElement * element)
   return TRUE;
 }
 
+/* Takes ownership of @caps and @props */
 static GstDevice *
 gst_pulse_device_new (guint device_index, const gchar * device_name,
-    GstCaps * caps, const gchar * internal_name, GstPulseDeviceType type)
+    GstCaps * caps, const gchar * internal_name, GstPulseDeviceType type,
+    GstStructure * props, gboolean is_default)
 {
   GstPulseDevice *gstdev;
   const gchar *element = NULL;
@@ -628,14 +720,18 @@ gst_pulse_device_new (guint device_index, const gchar * device_name,
       break;
   }
 
-
+  gst_structure_set (props, "is-default", G_TYPE_BOOLEAN, is_default, NULL);
   gstdev = g_object_new (GST_TYPE_PULSE_DEVICE,
       "display-name", device_name, "caps", caps, "device-class", klass,
-      "internal-name", internal_name, NULL);
+      "internal-name", internal_name, "properties", props, NULL);
 
   gstdev->type = type;
   gstdev->device_index = device_index;
   gstdev->element = element;
+  gstdev->is_default = is_default;
+
+  gst_structure_free (props);
+  gst_caps_unref (caps);
 
   return GST_DEVICE (gstdev);
 }
