@@ -17,7 +17,7 @@
  *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with gst-pulse; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
  *  USA.
  */
 
@@ -30,9 +30,6 @@
 #include <pulse/pulseaudio.h>
 #include <pulse/thread-mainloop.h>
 
-#include "pulsemixerctrl.h"
-#include "pulseprobe.h"
-
 G_BEGIN_DECLS
 
 #define GST_TYPE_PULSESRC \
@@ -69,19 +66,18 @@ struct _GstPulseSrc
   size_t read_buffer_length;
 
   gchar *device_description;
-  GstPulseMixerCtrl *mixer;
-  GstPulseProbe *probe;
 
-#ifdef HAVE_PULSE_1_0
   gdouble volume;
   gboolean volume_set:1;
   gboolean mute:1;
   gboolean mute_set:1;
+  guint32 current_source_idx;
+  gchar *current_source_name;
 
   gint notify; /* atomic */
-#endif
 
   gboolean corked:1;
+  gboolean stream_connected:1;
   gboolean operation_success:1;
   gboolean paused:1;
   gboolean in_read:1;
