@@ -37,7 +37,7 @@
 #include "config.h"
 #endif
 #include <unistd.h>
-#include <sys/poll.h>
+#include <poll.h>
 #include <sys/socket.h>
 #include <errno.h>
 #include <fcntl.h>
