@@ -114,10 +114,10 @@ gst_speex_dec_class_init (GstSpeexDecClass * klass)
       g_param_spec_boolean ("enh", "Enh", "Enable perceptual enhancement",
           DEFAULT_ENH, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&speex_dec_src_factory));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&speex_dec_sink_factory));
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &speex_dec_src_factory);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &speex_dec_sink_factory);
   gst_element_class_set_static_metadata (gstelement_class,
       "Speex audio decoder", "Codec/Decoder/Audio",
       "decode speex streams to audio", "Wim Taymans <wim@fluendo.com>");
