@@ -72,10 +72,10 @@ gst_cpu_report_class_init (GstCpuReportClass * g_class)
 
   gobject_class->finalize = gst_cpu_report_finalize;
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&cpu_report_sink_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&cpu_report_src_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &cpu_report_sink_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &cpu_report_src_template);
 
   gst_element_class_set_static_metadata (element_class, "CPU report",
       "Testing",
