@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -26,7 +26,7 @@
  * <refsect2>
  * <title>Example launch line</title>
  * |[
- * gst-launch filesrc location=song.ogg ! oggdemux ! vorbisdec ! audioconvert ! equalizer-10bands band2=3.0 ! alsasink
+ * gst-launch-1.0 filesrc location=song.ogg ! oggdemux ! vorbisdec ! audioconvert ! equalizer-10bands band2=3.0 ! alsasink
  * ]| This raises the volume of the 3rd band which is at 119 Hz by 3 db.
  * </refsect2>
  */
@@ -63,8 +63,8 @@ GST_DEBUG_CATEGORY_EXTERN (equalizer_debug);
 
 
 #define gst_iir_equalizer_10bands_parent_class parent_class
-G_DEFINE_TYPE_WITH_CODE (GstIirEqualizer10Bands, gst_iir_equalizer_10bands,
-    GST_TYPE_IIR_EQUALIZER, G_IMPLEMENT_INTERFACE (GST_TYPE_PRESET, NULL));
+G_DEFINE_TYPE (GstIirEqualizer10Bands, gst_iir_equalizer_10bands,
+    GST_TYPE_IIR_EQUALIZER);
 
 /* equalizer implementation */
 
@@ -146,38 +146,38 @@ static void
 gst_iir_equalizer_10bands_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
 {
-  GstIirEqualizer *equ = GST_IIR_EQUALIZER (object);
+  GstChildProxy *equ = GST_CHILD_PROXY (object);
 
   switch (prop_id) {
     case PROP_BAND0:
-      gst_child_proxy_set_property (G_OBJECT (equ), "band0::gain", value);
+      gst_child_proxy_set_property (equ, "band0::gain", value);
       break;
     case PROP_BAND1:
-      gst_child_proxy_set_property (G_OBJECT (equ), "band1::gain", value);
+      gst_child_proxy_set_property (equ, "band1::gain", value);
       break;
     case PROP_BAND2:
-      gst_child_proxy_set_property (G_OBJECT (equ), "band2::gain", value);
+      gst_child_proxy_set_property (equ, "band2::gain", value);
       break;
     case PROP_BAND3:
-      gst_child_proxy_set_property (G_OBJECT (equ), "band3::gain", value);
+      gst_child_proxy_set_property (equ, "band3::gain", value);
       break;
     case PROP_BAND4:
-      gst_child_proxy_set_property (G_OBJECT (equ), "band4::gain", value);
+      gst_child_proxy_set_property (equ, "band4::gain", value);
       break;
     case PROP_BAND5:
-      gst_child_proxy_set_property (G_OBJECT (equ), "band5::gain", value);
+      gst_child_proxy_set_property (equ, "band5::gain", value);
       break;
     case PROP_BAND6:
-      gst_child_proxy_set_property (G_OBJECT (equ), "band6::gain", value);
+      gst_child_proxy_set_property (equ, "band6::gain", value);
       break;
     case PROP_BAND7:
-      gst_child_proxy_set_property (G_OBJECT (equ), "band7::gain", value);
+      gst_child_proxy_set_property (equ, "band7::gain", value);
       break;
     case PROP_BAND8:
-      gst_child_proxy_set_property (G_OBJECT (equ), "band8::gain", value);
+      gst_child_proxy_set_property (equ, "band8::gain", value);
       break;
     case PROP_BAND9:
-      gst_child_proxy_set_property (G_OBJECT (equ), "band9::gain", value);
+      gst_child_proxy_set_property (equ, "band9::gain", value);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -189,38 +189,38 @@ static void
 gst_iir_equalizer_10bands_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec)
 {
-  GstIirEqualizer *equ = GST_IIR_EQUALIZER (object);
+  GstChildProxy *equ = GST_CHILD_PROXY (object);
 
   switch (prop_id) {
     case PROP_BAND0:
-      gst_child_proxy_get_property (G_OBJECT (equ), "band0::gain", value);
+      gst_child_proxy_get_property (equ, "band0::gain", value);
       break;
     case PROP_BAND1:
-      gst_child_proxy_get_property (G_OBJECT (equ), "band1::gain", value);
+      gst_child_proxy_get_property (equ, "band1::gain", value);
       break;
     case PROP_BAND2:
-      gst_child_proxy_get_property (G_OBJECT (equ), "band2::gain", value);
+      gst_child_proxy_get_property (equ, "band2::gain", value);
       break;
     case PROP_BAND3:
-      gst_child_proxy_get_property (G_OBJECT (equ), "band3::gain", value);
+      gst_child_proxy_get_property (equ, "band3::gain", value);
       break;
     case PROP_BAND4:
-      gst_child_proxy_get_property (G_OBJECT (equ), "band4::gain", value);
+      gst_child_proxy_get_property (equ, "band4::gain", value);
       break;
     case PROP_BAND5:
-      gst_child_proxy_get_property (G_OBJECT (equ), "band5::gain", value);
+      gst_child_proxy_get_property (equ, "band5::gain", value);
       break;
     case PROP_BAND6:
-      gst_child_proxy_get_property (G_OBJECT (equ), "band6::gain", value);
+      gst_child_proxy_get_property (equ, "band6::gain", value);
       break;
     case PROP_BAND7:
-      gst_child_proxy_get_property (G_OBJECT (equ), "band7::gain", value);
+      gst_child_proxy_get_property (equ, "band7::gain", value);
       break;
     case PROP_BAND8:
-      gst_child_proxy_get_property (G_OBJECT (equ), "band8::gain", value);
+      gst_child_proxy_get_property (equ, "band8::gain", value);
       break;
     case PROP_BAND9:
-      gst_child_proxy_get_property (G_OBJECT (equ), "band9::gain", value);
+      gst_child_proxy_get_property (equ, "band9::gain", value);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
