@@ -37,14 +37,6 @@ GST_DEBUG_CATEGORY_STATIC (splitmux_part_debug);
 #define SPLITMUX_PART_TYPE_LOCK(p) g_mutex_lock(&(p)->type_lock)
 #define SPLITMUX_PART_TYPE_UNLOCK(p) g_mutex_unlock(&(p)->type_lock)
 
-enum
-{
-  SIGNAL_PREPARED,
-  LAST_SIGNAL
-};
-
-static guint part_reader_signals[LAST_SIGNAL] = { 0 };
-
 typedef struct _GstSplitMuxPartPad
 {
   GstPad parent;
@@ -60,6 +52,7 @@ typedef struct _GstSplitMuxPartPad
   gboolean flushing;
   gboolean seen_buffer;
 
+  gboolean is_sparse;
   GstClockTime max_ts;
   GstSegment segment;
 
@@ -87,6 +80,10 @@ static void type_found (GstElement * typefind, guint probability,
     GstCaps * caps, GstSplitMuxPartReader * reader);
 static void check_if_pads_collected (GstSplitMuxPartReader * reader);
 
+static void
+gst_splitmux_part_reader_finish_measuring_streams (GstSplitMuxPartReader *
+    reader);
+
 /* Called with reader lock held */
 static gboolean
 have_empty_queue (GstSplitMuxPartReader * reader)
@@ -341,6 +338,12 @@ splitmux_part_pad_event (GstPad * pad, GstObject * parent, GstEvent * event)
     goto drop_event;
 
   switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_STREAM_START:{
+      GstStreamFlags flags;
+      gst_event_parse_stream_flags (event, &flags);
+      part_pad->is_sparse = (flags & GST_STREAM_FLAG_SPARSE);
+      break;
+    }
     case GST_EVENT_SEGMENT:{
       GstSegment *seg = &part_pad->segment;
 
@@ -408,7 +411,10 @@ splitmux_part_pad_event (GstPad * pad, GstObject * parent, GstEvent * event)
           GST_LOG_OBJECT (reader,
               "EOS while measuring streams. Resetting for ready");
           reader->prep_state = PART_STATE_PREPARING_RESET_FOR_READY;
-          SPLITMUX_PART_BROADCAST (reader);
+
+          gst_element_call_async (GST_ELEMENT_CAST (reader),
+              (GstElementCallAsyncFunc)
+              gst_splitmux_part_reader_finish_measuring_streams, NULL, NULL);
         }
         goto drop_event;
       }
@@ -449,12 +455,8 @@ splitmux_part_pad_event (GstPad * pad, GstObject * parent, GstEvent * event)
       GST_EVENT_TYPE (event) == GST_EVENT_FLUSH_STOP)
     goto drop_event;
 
-  if (!block_until_can_push (reader)) {
-    SPLITMUX_PART_UNLOCK (reader);
-    gst_object_unref (target);
-    gst_event_unref (event);
-    return FALSE;
-  }
+  if (!block_until_can_push (reader))
+    goto drop_event;
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_GAP:{
@@ -612,10 +614,6 @@ gst_splitmux_part_reader_class_init (GstSplitMuxPartReaderClass * klass)
   gobject_klass->dispose = splitmux_part_reader_dispose;
   gobject_klass->finalize = splitmux_part_reader_finalize;
 
-  part_reader_signals[SIGNAL_PREPARED] =
-      g_signal_new ("prepared", G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_FIRST, G_STRUCT_OFFSET (GstSplitMuxPartReaderClass,
-          prepared), NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
   gstelement_class->change_state = gst_splitmux_part_reader_change_state;
   gstelement_class->send_event = gst_splitmux_part_reader_send_event;
 
@@ -686,6 +684,37 @@ splitmux_part_reader_finalize (GObject * object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static void
+do_async_start (GstSplitMuxPartReader * reader)
+{
+  GstMessage *message;
+
+  GST_STATE_LOCK (reader);
+  reader->async_pending = TRUE;
+
+  message = gst_message_new_async_start (GST_OBJECT_CAST (reader));
+  GST_BIN_CLASS (parent_class)->handle_message (GST_BIN_CAST (reader), message);
+  GST_STATE_UNLOCK (reader);
+}
+
+static void
+do_async_done (GstSplitMuxPartReader * reader)
+{
+  GstMessage *message;
+
+  GST_STATE_LOCK (reader);
+  if (reader->async_pending) {
+    message =
+        gst_message_new_async_done (GST_OBJECT_CAST (reader),
+        GST_CLOCK_TIME_NONE);
+    GST_BIN_CLASS (parent_class)->handle_message (GST_BIN_CAST (reader),
+        message);
+
+    reader->async_pending = FALSE;
+  }
+  GST_STATE_UNLOCK (reader);
+}
+
 static void
 splitmux_part_reader_reset (GstSplitMuxPartReader * reader)
 {
@@ -816,12 +845,12 @@ gst_splitmux_part_reader_seek_to_time_locked (GstSplitMuxPartReader * reader,
 /* Map the passed segment to 'internal' time from 0 to length of this piece and seek. Lock cannot be held */
 static gboolean
 gst_splitmux_part_reader_seek_to_segment (GstSplitMuxPartReader * reader,
-    GstSegment * target_seg)
+    GstSegment * target_seg, GstSeekFlags extra_flags)
 {
   GstSeekFlags flags;
   GstClockTime start = 0, stop = GST_CLOCK_TIME_NONE;
 
-  flags = target_seg->flags | GST_SEEK_FLAG_FLUSH;
+  flags = target_seg->flags | GST_SEEK_FLAG_FLUSH | extra_flags;
 
   SPLITMUX_PART_LOCK (reader);
   if (target_seg->start >= reader->start_offset)
@@ -847,6 +876,7 @@ gst_splitmux_part_reader_seek_to_segment (GstSplitMuxPartReader * reader,
 static void
 gst_splitmux_part_reader_measure_streams (GstSplitMuxPartReader * reader)
 {
+  SPLITMUX_PART_LOCK (reader);
   /* Trigger a flushing seek to near the end of the file and run each stream
    * to EOS in order to find the smallest end timestamp to start the next
    * file from
@@ -856,18 +886,23 @@ gst_splitmux_part_reader_measure_streams (GstSplitMuxPartReader * reader)
     GstClockTime seek_ts = reader->duration - (0.5 * GST_SECOND);
     gst_splitmux_part_reader_seek_to_time_locked (reader, seek_ts);
   }
+  SPLITMUX_PART_UNLOCK (reader);
+}
 
-  /* Wait for things to happen */
-  while (reader->prep_state == PART_STATE_PREPARING_MEASURE_STREAMS)
-    SPLITMUX_PART_WAIT (reader);
-
+static void
+gst_splitmux_part_reader_finish_measuring_streams (GstSplitMuxPartReader *
+    reader)
+{
+  SPLITMUX_PART_LOCK (reader);
   if (reader->prep_state == PART_STATE_PREPARING_RESET_FOR_READY) {
     /* Fire the prepared signal and go to READY state */
     GST_DEBUG_OBJECT (reader,
-        "Stream measuring complete. File %s is now ready. Firing prepared signal",
-        reader->path);
+        "Stream measuring complete. File %s is now ready", reader->path);
     reader->prep_state = PART_STATE_READY;
-    g_signal_emit (reader, part_reader_signals[SIGNAL_PREPARED], 0, NULL);
+    SPLITMUX_PART_UNLOCK (reader);
+    do_async_done (reader);
+  } else {
+    SPLITMUX_PART_UNLOCK (reader);
   }
 }
 
@@ -936,7 +971,9 @@ check_if_pads_collected (GstSplitMuxPartReader * reader)
       GST_DEBUG_OBJECT (reader,
           "no more pads - file %s. Measuring stream length", reader->path);
       reader->prep_state = PART_STATE_PREPARING_MEASURE_STREAMS;
-      SPLITMUX_PART_BROADCAST (reader);
+      gst_element_call_async (GST_ELEMENT_CAST (reader),
+          (GstElementCallAsyncFunc) gst_splitmux_part_reader_measure_streams,
+          NULL, NULL);
     }
   }
 }
@@ -1038,13 +1075,16 @@ gst_splitmux_part_reader_change_state (GstElement * element,
       break;
     }
     case GST_STATE_CHANGE_READY_TO_PAUSED:{
-      /* Hold the splitmux type lock until after the
-       * parent state change function has finished
-       * changing the states of things, and type finding can continue */
       SPLITMUX_PART_LOCK (reader);
       g_object_set (reader->src, "location", reader->path, NULL);
+      reader->prep_state = PART_STATE_PREPARING_COLLECT_STREAMS;
+      gst_splitmux_part_reader_set_flushing_locked (reader, FALSE);
+      reader->running = TRUE;
       SPLITMUX_PART_UNLOCK (reader);
-      SPLITMUX_PART_TYPE_LOCK (reader);
+
+      /* we go to PAUSED asynchronously once all streams have been collected
+       * and seeks to measure the stream lengths are done */
+      do_async_start (reader);
       break;
     }
     case GST_STATE_CHANGE_READY_TO_NULL:
@@ -1068,36 +1108,16 @@ gst_splitmux_part_reader_change_state (GstElement * element,
 
   ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
   if (ret == GST_STATE_CHANGE_FAILURE) {
-    if (transition == GST_STATE_CHANGE_READY_TO_PAUSED) {
-      /* Make sure to release the lock we took above */
-      SPLITMUX_PART_TYPE_UNLOCK (reader);
-    }
+    do_async_done (reader);
     goto beach;
   }
 
   switch (transition) {
     case GST_STATE_CHANGE_READY_TO_PAUSED:
-      /* Sleep and wait until all streams have been collected, then do the seeks
-       * to measure the stream lengths. This took the type lock above,
-       * but it's OK to release it now and let typefinding happen... */
-      SPLITMUX_PART_TYPE_UNLOCK (reader);
-
-      SPLITMUX_PART_LOCK (reader);
-      reader->prep_state = PART_STATE_PREPARING_COLLECT_STREAMS;
-      gst_splitmux_part_reader_set_flushing_locked (reader, FALSE);
-      reader->running = TRUE;
-
-      while (reader->prep_state == PART_STATE_PREPARING_COLLECT_STREAMS) {
-        GST_LOG_OBJECT (reader, "Waiting to collect all output streams");
-        SPLITMUX_PART_WAIT (reader);
-      }
-
-      if (reader->prep_state == PART_STATE_PREPARING_MEASURE_STREAMS ||
-          reader->prep_state == PART_STATE_PREPARING_RESET_FOR_READY) {
-        gst_splitmux_part_reader_measure_streams (reader);
-      } else if (reader->prep_state == PART_STATE_FAILED)
-        ret = GST_STATE_CHANGE_FAILURE;
-      SPLITMUX_PART_UNLOCK (reader);
+      ret = GST_STATE_CHANGE_ASYNC;
+      break;
+    case GST_STATE_CHANGE_PAUSED_TO_READY:
+      do_async_done (reader);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       SPLITMUX_PART_LOCK (reader);
@@ -1118,28 +1138,6 @@ beach:
   return ret;
 }
 
-static gboolean
-check_bus_messages (GstSplitMuxPartReader * part)
-{
-  gboolean ret = FALSE;
-  GstBus *bus;
-  GstMessage *m;
-
-  bus = gst_element_get_bus (GST_ELEMENT_CAST (part));
-  while ((m = gst_bus_pop (bus)) != NULL) {
-    if (GST_MESSAGE_TYPE (m) == GST_MESSAGE_ERROR) {
-      GST_LOG_OBJECT (part, "Got error message while preparing. Failing.");
-      gst_message_unref (m);
-      goto done;
-    }
-    gst_message_unref (m);
-  }
-  ret = TRUE;
-done:
-  gst_object_unref (bus);
-  return ret;
-}
-
 gboolean
 gst_splitmux_part_reader_prepare (GstSplitMuxPartReader * part)
 {
@@ -1147,10 +1145,10 @@ gst_splitmux_part_reader_prepare (GstSplitMuxPartReader * part)
 
   ret = gst_element_set_state (GST_ELEMENT_CAST (part), GST_STATE_PAUSED);
 
-  if (ret != GST_STATE_CHANGE_SUCCESS)
+  if (ret == GST_STATE_CHANGE_FAILURE)
     return FALSE;
 
-  return check_bus_messages (part);
+  return TRUE;
 }
 
 void
@@ -1168,11 +1166,11 @@ gst_splitmux_part_reader_set_location (GstSplitMuxPartReader * reader,
 
 gboolean
 gst_splitmux_part_reader_activate (GstSplitMuxPartReader * reader,
-    GstSegment * seg)
+    GstSegment * seg, GstSeekFlags extra_flags)
 {
   GST_DEBUG_OBJECT (reader, "Activating part reader");
 
-  if (!gst_splitmux_part_reader_seek_to_segment (reader, seg)) {
+  if (!gst_splitmux_part_reader_seek_to_segment (reader, seg, extra_flags)) {
     GST_ERROR_OBJECT (reader, "Failed to seek part to %" GST_SEGMENT_FORMAT,
         seg);
     return FALSE;
@@ -1237,7 +1235,7 @@ gst_splitmux_part_reader_get_end_offset (GstSplitMuxPartReader * reader)
   SPLITMUX_PART_LOCK (reader);
   for (cur = g_list_first (reader->pads); cur != NULL; cur = g_list_next (cur)) {
     GstSplitMuxPartPad *part_pad = SPLITMUX_PART_PAD_CAST (cur->data);
-    if (part_pad->max_ts < ret)
+    if (!part_pad->is_sparse && part_pad->max_ts < ret)
       ret = part_pad->max_ts;
   }
 
@@ -1353,9 +1351,12 @@ bus_handler (GstBin * bin, GstMessage * message)
       /* Make sure to set the state to failed and wake up the listener
        * on error */
       SPLITMUX_PART_LOCK (reader);
+      GST_ERROR_OBJECT (reader, "Got error message from child %" GST_PTR_FORMAT
+          " marking this reader as failed", GST_MESSAGE_SRC (message));
       reader->prep_state = PART_STATE_FAILED;
       SPLITMUX_PART_BROADCAST (reader);
       SPLITMUX_PART_UNLOCK (reader);
+      do_async_done (reader);
       break;
     default:
       break;
