@@ -81,20 +81,26 @@ static void
 on_ssrc_active_cb (GstElement * rtpbin, guint sessid, guint ssrc,
     GstElement * depay)
 {
-  GObject *session, *isrc, *osrc;
+  GObject *session, *osrc;
 
   g_print ("got RTCP from session %u, SSRC %u\n", sessid, ssrc);
 
   /* get the right session */
   g_signal_emit_by_name (rtpbin, "get-internal-session", sessid, &session);
 
+#if 0
+  /* FIXME: This is broken in rtpbin */
   /* get the internal source (the SSRC allocated to us, the receiver */
   g_object_get (session, "internal-source", &isrc, NULL);
   print_source_stats (isrc);
+  g_object_unref (isrc);
+#endif
 
   /* get the remote source that sent us RTCP */
   g_signal_emit_by_name (session, "get-source-by-ssrc", ssrc, &osrc);
   print_source_stats (osrc);
+  g_object_unref (osrc);
+  g_object_unref (session);
 }
 
 /* will be called when rtpbin has validated a payload that we can depayload */
